import { reactive, readonly } from 'vue'
import axios from 'axios'
import { handleMsg, sendSub } from './utils/websocket'

const token = window.localStorage.getItem('token')

const server = process.env.VUE_APP_NUKE_URI
const api_proto = process.env.VUE_APP_NUKE_API_PROTO
const ws_proto = process.env.VUE_APP_NUKE_WS_PROTO

const state = reactive({
  name: '',
  token: token || '',
  session: '',
  loggedIn: false,
  ws: null,
  events: [],
  rosters: [],
  matches: [],
  casters: [],
  hosts: [],
  streams: [],
  streamsFull: [],
  brackets: [],
  overlayStream: null,
  server: false,
})

let interval = 0

const methods = {
  authenticate: async (token) => {
    state.session = null
    window.localStorage.setItem('token', token)
    return axios.post(`${api_proto}://${server}/api/v1`, {
      token,
    })
      .then((res) => {
        state.session = res.data.session
        state.name = res.data.name
        state.loggedIn = true
        return true
      })
  },
  connect: () => {
    if (state.session !== null) {
      state.ws = new WebSocket(`${ws_proto}://${server}/ws/${state.session}`)
      state.ws.onopen = () => {
        clearInterval(interval)
        sendSub(state.ws)
        state.ws.onclose = () => {
          console.warn('Lost connection to nuke. Attempting to reconnect every 5 seconds...')
          interval = setInterval(() => {
            methods.authenticate(state.token)
              .then(() => {
                methods.connect()
              })
              .catch((_) => {
                console.warn('Error reconnecting! Trying again')
              })
          }, 5000)
        }
      }
      state.ws.onmessage = (e) => {
        const data = JSON.parse(e.data)
        handleMsg(data)
      }
    }
  }
}

export default {
  state,
  methods,
}
